package com.winhc.repal.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.winhc.repal.constant.Constant;
import com.winhc.repal.entity.RepalRolePermission;
import com.winhc.repal.enums.DeletedStatusEnum;
import com.winhc.repal.enums.PermissionEnum;
import com.winhc.repal.model.dto.UpdateRolePermissionDTO;
import com.winhc.repal.model.vo.RolePermissionVO;
import com.winhc.repal.repository.RepalRolePermissionMapper;
import com.winhc.repal.service.RepalRolePermissionService;
import com.winhc.repal.util.RedisUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @description RepalRolePermission 接口实现类
 * @author Generator
 * @date 2022-04-08
 */
@Service
public class RepalRolePermissionServiceImpl extends ServiceImpl<RepalRolePermissionMapper, RepalRolePermission> implements RepalRolePermissionService {

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private RepalRolePermissionMapper repalRolePermissionMapper;


    @Override
    public Boolean checkUserPermission(Long userId, String code) {
        return getUserPermissionCodeSet(userId).contains(code);
    }

    @Override
    public Set<String> getUserPermissionCodeSet(Long userId) {
        Set<String> codeSet = redisUtil.setMembers(Constant.USER_PERMISSION_KEY + userId);
        if (CollUtil.isEmpty(codeSet)) {
            codeSet = repalRolePermissionMapper.getUserPermissionCodeSet(userId);
            if (CollUtil.isEmpty(codeSet)) {
                return new HashSet<>();
            }
            processUserPermissionCache(userId, codeSet);
        }
        return codeSet;
    }

    @Override
    public Boolean processUserPermissionCache(Long userId, Set<String> codeSet) {
        redisUtil.sAdd(Constant.USER_PERMISSION_KEY + userId, codeSet.toArray(new String[]{}));
        redisUtil.expire(Constant.USER_PERMISSION_KEY + userId, 7, TimeUnit.DAYS);
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean addRolePermission(Long roleId, List<String> perCodeList) {
        if (CollUtil.isEmpty(perCodeList)) {
            return true;
        }
        RepalRolePermission permission;
        for (String code : perCodeList) {
            permission = new RepalRolePermission();
            permission.setPermissionCode(code);
            permission.setPermissionName(Constant.PERMISSION_MAP.get(code));
            permission.setRepalGroupRoleId(roleId);
            this.save(permission);
        }
        return true;
    }

    @Override
    public List<RepalRolePermission> getRolePermissionList(Long roleId) {
        return this.list(Wrappers.lambdaQuery(RepalRolePermission.class)
                .eq(RepalRolePermission::getRepalGroupRoleId, roleId)
                .eq(RepalRolePermission::getDeleted, DeletedStatusEnum.NORMAL.getCode()));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateRolePermission(UpdateRolePermissionDTO dto) {
        Set<String> codeSet = new HashSet<>(repalRolePermissionMapper.getRolePermissionCodeList(dto.getRoleId()));
        List<String> disjunction = new ArrayList<>(CollUtil.disjunction(dto.getCodeSet(), codeSet));
        if (CollUtil.isEmpty(disjunction)) {
            // 无修改
            return true;
        }
        for (String code : disjunction) {
            if (codeSet.contains(code) && !dto.getCodeSet().contains(code)) {
                // remove
                this.update(Wrappers.lambdaUpdate(RepalRolePermission.class)
                        .eq(RepalRolePermission::getRepalGroupRoleId, dto.getRoleId())
                        .eq(RepalRolePermission::getPermissionCode, code)
                        .set(RepalRolePermission::getDeleted, DeletedStatusEnum.DELETE.getCode()));
            }
            if (!codeSet.contains(code) && dto.getCodeSet().contains(code)) {
                // add
                RepalRolePermission permission = new RepalRolePermission();
                permission.setPermissionCode(code);
                permission.setPermissionName(Constant.PERMISSION_MAP.get(code));
                permission.setRepalGroupRoleId(dto.getRoleId());
                this.save(permission);
            }
        }
        return true;
    }

    @Override
    public List<RolePermissionVO> getAllPermission() {
        return Arrays.stream(PermissionEnum.values()).map(t -> {
            RolePermissionVO vo = new RolePermissionVO();
            vo.setPermissionCode(t.getCode());
            vo.setPermissionName(t.getMsg());
            return vo;
        }).collect(Collectors.toList());
    }
}
